/* !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
   This file is built by embed.pl from data in embed.pl, pp.sym, intrpvar.h,
   perlvars.h and thrdvar.h.  Any changes made here will be lost!
*/

/* (Doing namespace management portably in C is really gross.) */

/* Put interpreter-specific symbols into a struct? */

#ifdef MULTIPLICITY

#ifndef USE_THREADS
/* If we do not have threads then per-thread vars are per-interpreter */

#ifdef PERL_IMPLICIT_CONTEXT

/* everything has an implicit context pointer */

#define PL_Sv			(my_perl->TSv)
#define PL_Xpv			(my_perl->TXpv)
#define PL_av_fetch_sv		(my_perl->Tav_fetch_sv)
#define PL_bodytarget		(my_perl->Tbodytarget)
#define PL_bostr		(my_perl->Tbostr)
#define PL_chopset		(my_perl->Tchopset)
#define PL_colors		(my_perl->Tcolors)
#define PL_colorset		(my_perl->Tcolorset)
#define PL_curcop		(my_perl->Tcurcop)
#define PL_curpad		(my_perl->Tcurpad)
#define PL_curpm		(my_perl->Tcurpm)
#define PL_curstack		(my_perl->Tcurstack)
#define PL_curstackinfo		(my_perl->Tcurstackinfo)
#define PL_curstash		(my_perl->Tcurstash)
#define PL_defoutgv		(my_perl->Tdefoutgv)
#define PL_defstash		(my_perl->Tdefstash)
#define PL_delaymagic		(my_perl->Tdelaymagic)
#define PL_dirty		(my_perl->Tdirty)
#define PL_dumpindent		(my_perl->Tdumpindent)
#define PL_extralen		(my_perl->Textralen)
#define PL_firstgv		(my_perl->Tfirstgv)
#define PL_formtarget		(my_perl->Tformtarget)
#define PL_hv_fetch_ent_mh	(my_perl->Thv_fetch_ent_mh)
#define PL_hv_fetch_sv		(my_perl->Thv_fetch_sv)
#define PL_in_eval		(my_perl->Tin_eval)
#define PL_last_in_gv		(my_perl->Tlast_in_gv)
#define PL_lastgotoprobe	(my_perl->Tlastgotoprobe)
#define PL_lastscream		(my_perl->Tlastscream)
#define PL_localizing		(my_perl->Tlocalizing)
#define PL_mainstack		(my_perl->Tmainstack)
#define PL_markstack		(my_perl->Tmarkstack)
#define PL_markstack_max	(my_perl->Tmarkstack_max)
#define PL_markstack_ptr	(my_perl->Tmarkstack_ptr)
#define PL_maxscream		(my_perl->Tmaxscream)
#define PL_modcount		(my_perl->Tmodcount)
#define PL_na			(my_perl->Tna)
#define PL_nrs			(my_perl->Tnrs)
#define PL_ofs			(my_perl->Tofs)
#define PL_ofslen		(my_perl->Tofslen)
#define PL_op			(my_perl->Top)
#define PL_opsave		(my_perl->Topsave)
#define PL_protect		(my_perl->Tprotect)
#define PL_reg_call_cc		(my_perl->Treg_call_cc)
#define PL_reg_curpm		(my_perl->Treg_curpm)
#define PL_reg_eval_set		(my_perl->Treg_eval_set)
#define PL_reg_flags		(my_perl->Treg_flags)
#define PL_reg_ganch		(my_perl->Treg_ganch)
#define PL_reg_leftiter		(my_perl->Treg_leftiter)
#define PL_reg_magic		(my_perl->Treg_magic)
#define PL_reg_maxiter		(my_perl->Treg_maxiter)
#define PL_reg_oldcurpm		(my_perl->Treg_oldcurpm)
#define PL_reg_oldpos		(my_perl->Treg_oldpos)
#define PL_reg_oldsaved		(my_perl->Treg_oldsaved)
#define PL_reg_oldsavedlen	(my_perl->Treg_oldsavedlen)
#define PL_reg_poscache		(my_perl->Treg_poscache)
#define PL_reg_poscache_size	(my_perl->Treg_poscache_size)
#define PL_reg_re		(my_perl->Treg_re)
#define PL_reg_start_tmp	(my_perl->Treg_start_tmp)
#define PL_reg_start_tmpl	(my_perl->Treg_start_tmpl)
#define PL_reg_starttry		(my_perl->Treg_starttry)
#define PL_reg_sv		(my_perl->Treg_sv)
#define PL_reg_whilem_seen	(my_perl->Treg_whilem_seen)
#define PL_regbol		(my_perl->Tregbol)
#define PL_regcc		(my_perl->Tregcc)
#define PL_regcode		(my_perl->Tregcode)
#define PL_regcomp_parse	(my_perl->Tregcomp_parse)
#define PL_regcomp_rx		(my_perl->Tregcomp_rx)
#define PL_regcompp		(my_perl->Tregcompp)
#define PL_regdata		(my_perl->Tregdata)
#define PL_regdummy		(my_perl->Tregdummy)
#define PL_regendp		(my_perl->Tregendp)
#define PL_regeol		(my_perl->Tregeol)
#define PL_regexecp		(my_perl->Tregexecp)
#define PL_regflags		(my_perl->Tregflags)
#define PL_regfree		(my_perl->Tregfree)
#define PL_regindent		(my_perl->Tregindent)
#define PL_reginput		(my_perl->Treginput)
#define PL_regint_start		(my_perl->Tregint_start)
#define PL_regint_string	(my_perl->Tregint_string)
#define PL_reginterp_cnt	(my_perl->Treginterp_cnt)
#define PL_reglastparen		(my_perl->Treglastparen)
#define PL_regnarrate		(my_perl->Tregnarrate)
#define PL_regnaughty		(my_perl->Tregnaughty)
#define PL_regnpar		(my_perl->Tregnpar)
#define PL_regprecomp		(my_perl->Tregprecomp)
#define PL_regprev		(my_perl->Tregprev)
#define PL_regprogram		(my_perl->Tregprogram)
#define PL_regsawback		(my_perl->Tregsawback)
#define PL_regseen		(my_perl->Tregseen)
#define PL_regsize		(my_perl->Tregsize)
#define PL_regstartp		(my_perl->Tregstartp)
#define PL_regtill		(my_perl->Tregtill)
#define PL_regxend		(my_perl->Tregxend)
#define PL_restartop		(my_perl->Trestartop)
#define PL_retstack		(my_perl->Tretstack)
#define PL_retstack_ix		(my_perl->Tretstack_ix)
#define PL_retstack_max		(my_perl->Tretstack_max)
#define PL_rs			(my_perl->Trs)
#define PL_savestack		(my_perl->Tsavestack)
#define PL_savestack_ix		(my_perl->Tsavestack_ix)
#define PL_savestack_max	(my_perl->Tsavestack_max)
#define PL_scopestack		(my_perl->Tscopestack)
#define PL_scopestack_ix	(my_perl->Tscopestack_ix)
#define PL_scopestack_max	(my_perl->Tscopestack_max)
#define PL_screamfirst		(my_perl->Tscreamfirst)
#define PL_screamnext		(my_perl->Tscreamnext)
#define PL_secondgv		(my_perl->Tsecondgv)
#define PL_seen_evals		(my_perl->Tseen_evals)
#define PL_seen_zerolen		(my_perl->Tseen_zerolen)
#define PL_sortcop		(my_perl->Tsortcop)
#define PL_sortcxix		(my_perl->Tsortcxix)
#define PL_sortstash		(my_perl->Tsortstash)
#define PL_stack_base		(my_perl->Tstack_base)
#define PL_stack_max		(my_perl->Tstack_max)
#define PL_stack_sp		(my_perl->Tstack_sp)
#define PL_start_env		(my_perl->Tstart_env)
#define PL_statbuf		(my_perl->Tstatbuf)
#define PL_statcache		(my_perl->Tstatcache)
#define PL_statgv		(my_perl->Tstatgv)
#define PL_statname		(my_perl->Tstatname)
#define PL_tainted		(my_perl->Ttainted)
#define PL_timesbuf		(my_perl->Ttimesbuf)
#define PL_tmps_floor		(my_perl->Ttmps_floor)
#define PL_tmps_ix		(my_perl->Ttmps_ix)
#define PL_tmps_max		(my_perl->Ttmps_max)
#define PL_tmps_stack		(my_perl->Ttmps_stack)
#define PL_top_env		(my_perl->Ttop_env)
#define PL_toptarget		(my_perl->Ttoptarget)
#define PL_watchaddr		(my_perl->Twatchaddr)
#define PL_watchok		(my_perl->Twatchok)

#else /* !PERL_IMPLICIT_CONTEXT */

/* traditional MULTIPLICITY (intepreter is in a global) */

#define PL_Sv			(PERL_GET_INTERP->TSv)
#define PL_Xpv			(PERL_GET_INTERP->TXpv)
#define PL_av_fetch_sv		(PERL_GET_INTERP->Tav_fetch_sv)
#define PL_bodytarget		(PERL_GET_INTERP->Tbodytarget)
#define PL_bostr		(PERL_GET_INTERP->Tbostr)
#define PL_chopset		(PERL_GET_INTERP->Tchopset)
#define PL_colors		(PERL_GET_INTERP->Tcolors)
#define PL_colorset		(PERL_GET_INTERP->Tcolorset)
#define PL_curcop		(PERL_GET_INTERP->Tcurcop)
#define PL_curpad		(PERL_GET_INTERP->Tcurpad)
#define PL_curpm		(PERL_GET_INTERP->Tcurpm)
#define PL_curstack		(PERL_GET_INTERP->Tcurstack)
#define PL_curstackinfo		(PERL_GET_INTERP->Tcurstackinfo)
#define PL_curstash		(PERL_GET_INTERP->Tcurstash)
#define PL_defoutgv		(PERL_GET_INTERP->Tdefoutgv)
#define PL_defstash		(PERL_GET_INTERP->Tdefstash)
#define PL_delaymagic		(PERL_GET_INTERP->Tdelaymagic)
#define PL_dirty		(PERL_GET_INTERP->Tdirty)
#define PL_dumpindent		(PERL_GET_INTERP->Tdumpindent)
#define PL_extralen		(PERL_GET_INTERP->Textralen)
#define PL_firstgv		(PERL_GET_INTERP->Tfirstgv)
#define PL_formtarget		(PERL_GET_INTERP->Tformtarget)
#define PL_hv_fetch_ent_mh	(PERL_GET_INTERP->Thv_fetch_ent_mh)
#define PL_hv_fetch_sv		(PERL_GET_INTERP->Thv_fetch_sv)
#define PL_in_eval		(PERL_GET_INTERP->Tin_eval)
#define PL_last_in_gv		(PERL_GET_INTERP->Tlast_in_gv)
#define PL_lastgotoprobe	(PERL_GET_INTERP->Tlastgotoprobe)
#define PL_lastscream		(PERL_GET_INTERP->Tlastscream)
#define PL_localizing		(PERL_GET_INTERP->Tlocalizing)
#define PL_mainstack		(PERL_GET_INTERP->Tmainstack)
#define PL_markstack		(PERL_GET_INTERP->Tmarkstack)
#define PL_markstack_max	(PERL_GET_INTERP->Tmarkstack_max)
#define PL_markstack_ptr	(PERL_GET_INTERP->Tmarkstack_ptr)
#define PL_maxscream		(PERL_GET_INTERP->Tmaxscream)
#define PL_modcount		(PERL_GET_INTERP->Tmodcount)
#define PL_na			(PERL_GET_INTERP->Tna)
#define PL_nrs			(PERL_GET_INTERP->Tnrs)
#define PL_ofs			(PERL_GET_INTERP->Tofs)
#define PL_ofslen		(PERL_GET_INTERP->Tofslen)
#define PL_op			(PERL_GET_INTERP->Top)
#define PL_opsave		(PERL_GET_INTERP->Topsave)
#define PL_protect		(PERL_GET_INTERP->Tprotect)
#define PL_reg_call_cc		(PERL_GET_INTERP->Treg_call_cc)
#define PL_reg_curpm		(PERL_GET_INTERP->Treg_curpm)
#define PL_reg_eval_set		(PERL_GET_INTERP->Treg_eval_set)
#define PL_reg_flags		(PERL_GET_INTERP->Treg_flags)
#define PL_reg_ganch		(PERL_GET_INTERP->Treg_ganch)
#define PL_reg_leftiter		(PERL_GET_INTERP->Treg_leftiter)
#define PL_reg_magic		(PERL_GET_INTERP->Treg_magic)
#define PL_reg_maxiter		(PERL_GET_INTERP->Treg_maxiter)
#define PL_reg_oldcurpm		(PERL_GET_INTERP->Treg_oldcurpm)
#define PL_reg_oldpos		(PERL_GET_INTERP->Treg_oldpos)
#define PL_reg_oldsaved		(PERL_GET_INTERP->Treg_oldsaved)
#define PL_reg_oldsavedlen	(PERL_GET_INTERP->Treg_oldsavedlen)
#define PL_reg_poscache		(PERL_GET_INTERP->Treg_poscache)
#define PL_reg_poscache_size	(PERL_GET_INTERP->Treg_poscache_size)
#define PL_reg_re		(PERL_GET_INTERP->Treg_re)
#define PL_reg_start_tmp	(PERL_GET_INTERP->Treg_start_tmp)
#define PL_reg_start_tmpl	(PERL_GET_INTERP->Treg_start_tmpl)
#define PL_reg_starttry		(PERL_GET_INTERP->Treg_starttry)
#define PL_reg_sv		(PERL_GET_INTERP->Treg_sv)
#define PL_reg_whilem_seen	(PERL_GET_INTERP->Treg_whilem_seen)
#define PL_regbol		(PERL_GET_INTERP->Tregbol)
#define PL_regcc		(PERL_GET_INTERP->Tregcc)
#define PL_regcode		(PERL_GET_INTERP->Tregcode)
#define PL_regcomp_parse	(PERL_GET_INTERP->Tregcomp_parse)
#define PL_regcomp_rx		(PERL_GET_INTERP->Tregcomp_rx)
#define PL_regcompp		(PERL_GET_INTERP->Tregcompp)
#define PL_regdata		(PERL_GET_INTERP->Tregdata)
#define PL_regdummy		(PERL_GET_INTERP->Tregdummy)
#define PL_regendp		(PERL_GET_INTERP->Tregendp)
#define PL_regeol		(PERL_GET_INTERP->Tregeol)
#define PL_regexecp		(PERL_GET_INTERP->Tregexecp)
#define PL_regflags		(PERL_GET_INTERP->Tregflags)
#define PL_regfree		(PERL_GET_INTERP->Tregfree)
#define PL_regindent		(PERL_GET_INTERP->Tregindent)
#define PL_reginput		(PERL_GET_INTERP->Treginput)
#define PL_regint_start		(PERL_GET_INTERP->Tregint_start)
#define PL_regint_string	(PERL_GET_INTERP->Tregint_string)
#define PL_reginterp_cnt	(PERL_GET_INTERP->Treginterp_cnt)
#define PL_reglastparen		(PERL_GET_INTERP->Treglastparen)
#define PL_regnarrate		(PERL_GET_INTERP->Tregnarrate)
#define PL_regnaughty		(PERL_GET_INTERP->Tregnaughty)
#define PL_regnpar		(PERL_GET_INTERP->Tregnpar)
#define PL_regprecomp		(PERL_GET_INTERP->Tregprecomp)
#define PL_regprev		(PERL_GET_INTERP->Tregprev)
#define PL_regprogram		(PERL_GET_INTERP->Tregprogram)
#define PL_regsawback		(PERL_GET_INTERP->Tregsawback)
#define PL_regseen		(PERL_GET_INTERP->Tregseen)
#define PL_regsize		(PERL_GET_INTERP->Tregsize)
#define PL_regstartp		(PERL_GET_INTERP->Tregstartp)
#define PL_regtill		(PERL_GET_INTERP->Tregtill)
#define PL_regxend		(PERL_GET_INTERP->Tregxend)
#define PL_restartop		(PERL_GET_INTERP->Trestartop)
#define PL_retstack		(PERL_GET_INTERP->Tretstack)
#define PL_retstack_ix		(PERL_GET_INTERP->Tretstack_ix)
#define PL_retstack_max		(PERL_GET_INTERP->Tretstack_max)
#define PL_rs			(PERL_GET_INTERP->Trs)
#define PL_savestack		(PERL_GET_INTERP->Tsavestack)
#define PL_savestack_ix		(PERL_GET_INTERP->Tsavestack_ix)
#define PL_savestack_max	(PERL_GET_INTERP->Tsavestack_max)
#define PL_scopestack		(PERL_GET_INTERP->Tscopestack)
#define PL_scopestack_ix	(PERL_GET_INTERP->Tscopestack_ix)
#define PL_scopestack_max	(PERL_GET_INTERP->Tscopestack_max)
#define PL_screamfirst		(PERL_GET_INTERP->Tscreamfirst)
#define PL_screamnext		(PERL_GET_INTERP->Tscreamnext)
#define PL_secondgv		(PERL_GET_INTERP->Tsecondgv)
#define PL_seen_evals		(PERL_GET_INTERP->Tseen_evals)
#define PL_seen_zerolen		(PERL_GET_INTERP->Tseen_zerolen)
#define PL_sortcop		(PERL_GET_INTERP->Tsortcop)
#define PL_sortcxix		(PERL_GET_INTERP->Tsortcxix)
#define PL_sortstash		(PERL_GET_INTERP->Tsortstash)
#define PL_stack_base		(PERL_GET_INTERP->Tstack_base)
#define PL_stack_max		(PERL_GET_INTERP->Tstack_max)
#define PL_stack_sp		(PERL_GET_INTERP->Tstack_sp)
#define PL_start_env		(PERL_GET_INTERP->Tstart_env)
#define PL_statbuf		(PERL_GET_INTERP->Tstatbuf)
#define PL_statcache		(PERL_GET_INTERP->Tstatcache)
#define PL_statgv		(PERL_GET_INTERP->Tstatgv)
#define PL_statname		(PERL_GET_INTERP->Tstatname)
#define PL_tainted		(PERL_GET_INTERP->Ttainted)
#define PL_timesbuf		(PERL_GET_INTERP->Ttimesbuf)
#define PL_tmps_floor		(PERL_GET_INTERP->Ttmps_floor)
#define PL_tmps_ix		(PERL_GET_INTERP->Ttmps_ix)
#define PL_tmps_max		(PERL_GET_INTERP->Ttmps_max)
#define PL_tmps_stack		(PERL_GET_INTERP->Ttmps_stack)
#define PL_top_env		(PERL_GET_INTERP->Ttop_env)
#define PL_toptarget		(PERL_GET_INTERP->Ttoptarget)
#define PL_watchaddr		(PERL_GET_INTERP->Twatchaddr)
#define PL_watchok		(PERL_GET_INTERP->Twatchok)

#endif /* !PERL_IMPLICIT_CONTEXT */
#endif /* !USE_THREADS */

/* These are always per-interpreter if there is more than one */

#define PL_Argv			(PERL_GET_INTERP->IArgv)
#define PL_Cmd			(PERL_GET_INTERP->ICmd)
#define PL_DBcv			(PERL_GET_INTERP->IDBcv)
#define PL_DBgv			(PERL_GET_INTERP->IDBgv)
#define PL_DBline		(PERL_GET_INTERP->IDBline)
#define PL_DBsignal		(PERL_GET_INTERP->IDBsignal)
#define PL_DBsingle		(PERL_GET_INTERP->IDBsingle)
#define PL_DBsub		(PERL_GET_INTERP->IDBsub)
#define PL_DBtrace		(PERL_GET_INTERP->IDBtrace)
#define PL_Dir			(PERL_GET_INTERP->IDir)
#define PL_Env			(PERL_GET_INTERP->IEnv)
#define PL_LIO			(PERL_GET_INTERP->ILIO)
#define PL_Mem			(PERL_GET_INTERP->IMem)
#define PL_Proc			(PERL_GET_INTERP->IProc)
#define PL_Sock			(PERL_GET_INTERP->ISock)
#define PL_StdIO		(PERL_GET_INTERP->IStdIO)
#define PL_amagic_generation	(PERL_GET_INTERP->Iamagic_generation)
#define PL_ampergv		(PERL_GET_INTERP->Iampergv)
#define PL_an			(PERL_GET_INTERP->Ian)
#define PL_archpat_auto		(PERL_GET_INTERP->Iarchpat_auto)
#define PL_argvgv		(PERL_GET_INTERP->Iargvgv)
#define PL_argvoutgv		(PERL_GET_INTERP->Iargvoutgv)
#define PL_basetime		(PERL_GET_INTERP->Ibasetime)
#define PL_beginav		(PERL_GET_INTERP->Ibeginav)
#define PL_bitcount		(PERL_GET_INTERP->Ibitcount)
#define PL_bufend		(PERL_GET_INTERP->Ibufend)
#define PL_bufptr		(PERL_GET_INTERP->Ibufptr)
#define PL_cddir		(PERL_GET_INTERP->Icddir)
#define PL_collation_ix		(PERL_GET_INTERP->Icollation_ix)
#define PL_collation_name	(PERL_GET_INTERP->Icollation_name)
#define PL_collation_standard	(PERL_GET_INTERP->Icollation_standard)
#define PL_collxfrm_base	(PERL_GET_INTERP->Icollxfrm_base)
#define PL_collxfrm_mult	(PERL_GET_INTERP->Icollxfrm_mult)
#define PL_compcv		(PERL_GET_INTERP->Icompcv)
#define PL_compiling		(PERL_GET_INTERP->Icompiling)
#define PL_comppad		(PERL_GET_INTERP->Icomppad)
#define PL_comppad_name		(PERL_GET_INTERP->Icomppad_name)
#define PL_comppad_name_fill	(PERL_GET_INTERP->Icomppad_name_fill)
#define PL_comppad_name_floor	(PERL_GET_INTERP->Icomppad_name_floor)
#define PL_cop_seqmax		(PERL_GET_INTERP->Icop_seqmax)
#define PL_copline		(PERL_GET_INTERP->Icopline)
#define PL_cred_mutex		(PERL_GET_INTERP->Icred_mutex)
#define PL_cryptseen		(PERL_GET_INTERP->Icryptseen)
#define PL_cshlen		(PERL_GET_INTERP->Icshlen)
#define PL_cshname		(PERL_GET_INTERP->Icshname)
#define PL_curcopdb		(PERL_GET_INTERP->Icurcopdb)
#define PL_curstname		(PERL_GET_INTERP->Icurstname)
#define PL_curthr		(PERL_GET_INTERP->Icurthr)
#define PL_dbargs		(PERL_GET_INTERP->Idbargs)
#define PL_debdelim		(PERL_GET_INTERP->Idebdelim)
#define PL_debname		(PERL_GET_INTERP->Idebname)
#define PL_debstash		(PERL_GET_INTERP->Idebstash)
#define PL_debug		(PERL_GET_INTERP->Idebug)
#define PL_defgv		(PERL_GET_INTERP->Idefgv)
#define PL_diehook		(PERL_GET_INTERP->Idiehook)
#define PL_dlevel		(PERL_GET_INTERP->Idlevel)
#define PL_dlmax		(PERL_GET_INTERP->Idlmax)
#define PL_doextract		(PERL_GET_INTERP->Idoextract)
#define PL_doswitches		(PERL_GET_INTERP->Idoswitches)
#define PL_dowarn		(PERL_GET_INTERP->Idowarn)
#define PL_e_script		(PERL_GET_INTERP->Ie_script)
#define PL_efloatbuf		(PERL_GET_INTERP->Iefloatbuf)
#define PL_efloatsize		(PERL_GET_INTERP->Iefloatsize)
#define PL_egid			(PERL_GET_INTERP->Iegid)
#define PL_endav		(PERL_GET_INTERP->Iendav)
#define PL_envgv		(PERL_GET_INTERP->Ienvgv)
#define PL_errgv		(PERL_GET_INTERP->Ierrgv)
#define PL_error_count		(PERL_GET_INTERP->Ierror_count)
#define PL_euid			(PERL_GET_INTERP->Ieuid)
#define PL_eval_cond		(PERL_GET_INTERP->Ieval_cond)
#define PL_eval_mutex		(PERL_GET_INTERP->Ieval_mutex)
#define PL_eval_owner		(PERL_GET_INTERP->Ieval_owner)
#define PL_eval_root		(PERL_GET_INTERP->Ieval_root)
#define PL_eval_start		(PERL_GET_INTERP->Ieval_start)
#define PL_evalseq		(PERL_GET_INTERP->Ievalseq)
#define PL_exitlist		(PERL_GET_INTERP->Iexitlist)
#define PL_exitlistlen		(PERL_GET_INTERP->Iexitlistlen)
#define PL_expect		(PERL_GET_INTERP->Iexpect)
#define PL_fdpid		(PERL_GET_INTERP->Ifdpid)
#define PL_filemode		(PERL_GET_INTERP->Ifilemode)
#define PL_filter_debug		(PERL_GET_INTERP->Ifilter_debug)
#define PL_forkprocess		(PERL_GET_INTERP->Iforkprocess)
#define PL_formfeed		(PERL_GET_INTERP->Iformfeed)
#define PL_generation		(PERL_GET_INTERP->Igeneration)
#define PL_gensym		(PERL_GET_INTERP->Igensym)
#define PL_gid			(PERL_GET_INTERP->Igid)
#define PL_glob_index		(PERL_GET_INTERP->Iglob_index)
#define PL_globalstash		(PERL_GET_INTERP->Iglobalstash)
#define PL_he_root		(PERL_GET_INTERP->Ihe_root)
#define PL_hintgv		(PERL_GET_INTERP->Ihintgv)
#define PL_hints		(PERL_GET_INTERP->Ihints)
#define PL_in_clean_all		(PERL_GET_INTERP->Iin_clean_all)
#define PL_in_clean_objs	(PERL_GET_INTERP->Iin_clean_objs)
#define PL_in_my		(PERL_GET_INTERP->Iin_my)
#define PL_in_my_stash		(PERL_GET_INTERP->Iin_my_stash)
#define PL_incgv		(PERL_GET_INTERP->Iincgv)
#define PL_initav		(PERL_GET_INTERP->Iinitav)
#define PL_inplace		(PERL_GET_INTERP->Iinplace)
#define PL_last_lop		(PERL_GET_INTERP->Ilast_lop)
#define PL_last_lop_op		(PERL_GET_INTERP->Ilast_lop_op)
#define PL_last_swash_hv	(PERL_GET_INTERP->Ilast_swash_hv)
#define PL_last_swash_key	(PERL_GET_INTERP->Ilast_swash_key)
#define PL_last_swash_klen	(PERL_GET_INTERP->Ilast_swash_klen)
#define PL_last_swash_slen	(PERL_GET_INTERP->Ilast_swash_slen)
#define PL_last_swash_tmps	(PERL_GET_INTERP->Ilast_swash_tmps)
#define PL_last_uni		(PERL_GET_INTERP->Ilast_uni)
#define PL_lastfd		(PERL_GET_INTERP->Ilastfd)
#define PL_lastsize		(PERL_GET_INTERP->Ilastsize)
#define PL_lastspbase		(PERL_GET_INTERP->Ilastspbase)
#define PL_laststatval		(PERL_GET_INTERP->Ilaststatval)
#define PL_laststype		(PERL_GET_INTERP->Ilaststype)
#define PL_leftgv		(PERL_GET_INTERP->Ileftgv)
#define PL_lex_brackets		(PERL_GET_INTERP->Ilex_brackets)
#define PL_lex_brackstack	(PERL_GET_INTERP->Ilex_brackstack)
#define PL_lex_casemods		(PERL_GET_INTERP->Ilex_casemods)
#define PL_lex_casestack	(PERL_GET_INTERP->Ilex_casestack)
#define PL_lex_defer		(PERL_GET_INTERP->Ilex_defer)
#define PL_lex_dojoin		(PERL_GET_INTERP->Ilex_dojoin)
#define PL_lex_expect		(PERL_GET_INTERP->Ilex_expect)
#define PL_lex_fakebrack	(PERL_GET_INTERP->Ilex_fakebrack)
#define PL_lex_formbrack	(PERL_GET_INTERP->Ilex_formbrack)
#define PL_lex_inpat		(PERL_GET_INTERP->Ilex_inpat)
#define PL_lex_inwhat		(PERL_GET_INTERP->Ilex_inwhat)
#define PL_lex_op		(PERL_GET_INTERP->Ilex_op)
#define PL_lex_repl		(PERL_GET_INTERP->Ilex_repl)
#define PL_lex_starts		(PERL_GET_INTERP->Ilex_starts)
#define PL_lex_state		(PERL_GET_INTERP->Ilex_state)
#define PL_lex_stuff		(PERL_GET_INTERP->Ilex_stuff)
#define PL_lineary		(PERL_GET_INTERP->Ilineary)
#define PL_linestart		(PERL_GET_INTERP->Ilinestart)
#define PL_linestr		(PERL_GET_INTERP->Ilinestr)
#define PL_localpatches		(PERL_GET_INTERP->Ilocalpatches)
#define PL_main_cv		(PERL_GET_INTERP->Imain_cv)
#define PL_main_root		(PERL_GET_INTERP->Imain_root)
#define PL_main_start		(PERL_GET_INTERP->Imain_start)
#define PL_malloc_mutex		(PERL_GET_INTERP->Imalloc_mutex)
#define PL_max_intro_pending	(PERL_GET_INTERP->Imax_intro_pending)
#define PL_maxo			(PERL_GET_INTERP->Imaxo)
#define PL_maxsysfd		(PERL_GET_INTERP->Imaxsysfd)
#define PL_mess_sv		(PERL_GET_INTERP->Imess_sv)
#define PL_min_intro_pending	(PERL_GET_INTERP->Imin_intro_pending)
#define PL_minus_F		(PERL_GET_INTERP->Iminus_F)
#define PL_minus_a		(PERL_GET_INTERP->Iminus_a)
#define PL_minus_c		(PERL_GET_INTERP->Iminus_c)
#define PL_minus_l		(PERL_GET_INTERP->Iminus_l)
#define PL_minus_n		(PERL_GET_INTERP->Iminus_n)
#define PL_minus_p		(PERL_GET_INTERP->Iminus_p)
#define PL_modglobal		(PERL_GET_INTERP->Imodglobal)
#define PL_multi_close		(PERL_GET_INTERP->Imulti_close)
#define PL_multi_end		(PERL_GET_INTERP->Imulti_end)
#define PL_multi_open		(PERL_GET_INTERP->Imulti_open)
#define PL_multi_start		(PERL_GET_INTERP->Imulti_start)
#define PL_multiline		(PERL_GET_INTERP->Imultiline)
#define PL_mystrk		(PERL_GET_INTERP->Imystrk)
#define PL_nexttoke		(PERL_GET_INTERP->Inexttoke)
#define PL_nexttype		(PERL_GET_INTERP->Inexttype)
#define PL_nextval		(PERL_GET_INTERP->Inextval)
#define PL_nice_chunk		(PERL_GET_INTERP->Inice_chunk)
#define PL_nice_chunk_size	(PERL_GET_INTERP->Inice_chunk_size)
#define PL_nomemok		(PERL_GET_INTERP->Inomemok)
#define PL_nthreads		(PERL_GET_INTERP->Inthreads)
#define PL_nthreads_cond	(PERL_GET_INTERP->Inthreads_cond)
#define PL_numeric_local	(PERL_GET_INTERP->Inumeric_local)
#define PL_numeric_name		(PERL_GET_INTERP->Inumeric_name)
#define PL_numeric_radix	(PERL_GET_INTERP->Inumeric_radix)
#define PL_numeric_standard	(PERL_GET_INTERP->Inumeric_standard)
#define PL_ofmt			(PERL_GET_INTERP->Iofmt)
#define PL_oldbufptr		(PERL_GET_INTERP->Ioldbufptr)
#define PL_oldlastpm		(PERL_GET_INTERP->Ioldlastpm)
#define PL_oldname		(PERL_GET_INTERP->Ioldname)
#define PL_oldoldbufptr		(PERL_GET_INTERP->Ioldoldbufptr)
#define PL_op_mask		(PERL_GET_INTERP->Iop_mask)
#define PL_op_seqmax		(PERL_GET_INTERP->Iop_seqmax)
#define PL_origalen		(PERL_GET_INTERP->Iorigalen)
#define PL_origargc		(PERL_GET_INTERP->Iorigargc)
#define PL_origargv		(PERL_GET_INTERP->Iorigargv)
#define PL_origenviron		(PERL_GET_INTERP->Iorigenviron)
#define PL_origfilename		(PERL_GET_INTERP->Iorigfilename)
#define PL_ors			(PERL_GET_INTERP->Iors)
#define PL_orslen		(PERL_GET_INTERP->Iorslen)
#define PL_osname		(PERL_GET_INTERP->Iosname)
#define PL_pad_reset_pending	(PERL_GET_INTERP->Ipad_reset_pending)
#define PL_padix		(PERL_GET_INTERP->Ipadix)
#define PL_padix_floor		(PERL_GET_INTERP->Ipadix_floor)
#define PL_parsehook		(PERL_GET_INTERP->Iparsehook)
#define PL_patchlevel		(PERL_GET_INTERP->Ipatchlevel)
#define PL_pending_ident	(PERL_GET_INTERP->Ipending_ident)
#define PL_perl_destruct_level	(PERL_GET_INTERP->Iperl_destruct_level)
#define PL_perldb		(PERL_GET_INTERP->Iperldb)
#define PL_pidstatus		(PERL_GET_INTERP->Ipidstatus)
#define PL_preambleav		(PERL_GET_INTERP->Ipreambleav)
#define PL_preambled		(PERL_GET_INTERP->Ipreambled)
#define PL_preprocess		(PERL_GET_INTERP->Ipreprocess)
#define PL_profiledata		(PERL_GET_INTERP->Iprofiledata)
#define PL_replgv		(PERL_GET_INTERP->Ireplgv)
#define PL_rightgv		(PERL_GET_INTERP->Irightgv)
#define PL_rsfp			(PERL_GET_INTERP->Irsfp)
#define PL_rsfp_filters		(PERL_GET_INTERP->Irsfp_filters)
#define PL_runops		(PERL_GET_INTERP->Irunops)
#define PL_sawampersand		(PERL_GET_INTERP->Isawampersand)
#define PL_sawstudy		(PERL_GET_INTERP->Isawstudy)
#define PL_sawvec		(PERL_GET_INTERP->Isawvec)
#define PL_sh_path		(PERL_GET_INTERP->Ish_path)
#define PL_siggv		(PERL_GET_INTERP->Isiggv)
#define PL_sighandlerp		(PERL_GET_INTERP->Isighandlerp)
#define PL_splitstr		(PERL_GET_INTERP->Isplitstr)
#define PL_srand_called		(PERL_GET_INTERP->Isrand_called)
#define PL_statusvalue		(PERL_GET_INTERP->Istatusvalue)
#define PL_statusvalue_vms	(PERL_GET_INTERP->Istatusvalue_vms)
#define PL_stdingv		(PERL_GET_INTERP->Istdingv)
#define PL_strchop		(PERL_GET_INTERP->Istrchop)
#define PL_strtab		(PERL_GET_INTERP->Istrtab)
#define PL_strtab_mutex		(PERL_GET_INTERP->Istrtab_mutex)
#define PL_sub_generation	(PERL_GET_INTERP->Isub_generation)
#define PL_sublex_info		(PERL_GET_INTERP->Isublex_info)
#define PL_subline		(PERL_GET_INTERP->Isubline)
#define PL_subname		(PERL_GET_INTERP->Isubname)
#define PL_sv_arenaroot		(PERL_GET_INTERP->Isv_arenaroot)
#define PL_sv_count		(PERL_GET_INTERP->Isv_count)
#define PL_sv_mutex		(PERL_GET_INTERP->Isv_mutex)
#define PL_sv_no		(PERL_GET_INTERP->Isv_no)
#define PL_sv_objcount		(PERL_GET_INTERP->Isv_objcount)
#define PL_sv_root		(PERL_GET_INTERP->Isv_root)
#define PL_sv_undef		(PERL_GET_INTERP->Isv_undef)
#define PL_sv_yes		(PERL_GET_INTERP->Isv_yes)
#define PL_svref_mutex		(PERL_GET_INTERP->Isvref_mutex)
#define PL_sys_intern		(PERL_GET_INTERP->Isys_intern)
#define PL_tainting		(PERL_GET_INTERP->Itainting)
#define PL_thisexpr		(PERL_GET_INTERP->Ithisexpr)
#define PL_thr_key		(PERL_GET_INTERP->Ithr_key)
#define PL_threadnum		(PERL_GET_INTERP->Ithreadnum)
#define PL_threads_mutex	(PERL_GET_INTERP->Ithreads_mutex)
#define PL_threadsv_names	(PERL_GET_INTERP->Ithreadsv_names)
#define PL_thrsv		(PERL_GET_INTERP->Ithrsv)
#define PL_tokenbuf		(PERL_GET_INTERP->Itokenbuf)
#define PL_uid			(PERL_GET_INTERP->Iuid)
#define PL_unsafe		(PERL_GET_INTERP->Iunsafe)
#define PL_utf8_alnum		(PERL_GET_INTERP->Iutf8_alnum)
#define PL_utf8_alnumc		(PERL_GET_INTERP->Iutf8_alnumc)
#define PL_utf8_alpha		(PERL_GET_INTERP->Iutf8_alpha)
#define PL_utf8_ascii		(PERL_GET_INTERP->Iutf8_ascii)
#define PL_utf8_cntrl		(PERL_GET_INTERP->Iutf8_cntrl)
#define PL_utf8_digit		(PERL_GET_INTERP->Iutf8_digit)
#define PL_utf8_graph		(PERL_GET_INTERP->Iutf8_graph)
#define PL_utf8_lower		(PERL_GET_INTERP->Iutf8_lower)
#define PL_utf8_mark		(PERL_GET_INTERP->Iutf8_mark)
#define PL_utf8_print		(PERL_GET_INTERP->Iutf8_print)
#define PL_utf8_punct		(PERL_GET_INTERP->Iutf8_punct)
#define PL_utf8_space		(PERL_GET_INTERP->Iutf8_space)
#define PL_utf8_tolower		(PERL_GET_INTERP->Iutf8_tolower)
#define PL_utf8_totitle		(PERL_GET_INTERP->Iutf8_totitle)
#define PL_utf8_toupper		(PERL_GET_INTERP->Iutf8_toupper)
#define PL_utf8_upper		(PERL_GET_INTERP->Iutf8_upper)
#define PL_utf8_xdigit		(PERL_GET_INTERP->Iutf8_xdigit)
#define PL_uudmap		(PERL_GET_INTERP->Iuudmap)
#define PL_warnhook		(PERL_GET_INTERP->Iwarnhook)
#define PL_xiv_arenaroot	(PERL_GET_INTERP->Ixiv_arenaroot)
#define PL_xiv_root		(PERL_GET_INTERP->Ixiv_root)
#define PL_xnv_root		(PERL_GET_INTERP->Ixnv_root)
#define PL_xpv_root		(PERL_GET_INTERP->Ixpv_root)
#define PL_xrv_root		(PERL_GET_INTERP->Ixrv_root)
#define PL_yychar		(PERL_GET_INTERP->Iyychar)
#define PL_yydebug		(PERL_GET_INTERP->Iyydebug)
#define PL_yyerrflag		(PERL_GET_INTERP->Iyyerrflag)
#define PL_yylval		(PERL_GET_INTERP->Iyylval)
#define PL_yynerrs		(PERL_GET_INTERP->Iyynerrs)
#define PL_yyval		(PERL_GET_INTERP->Iyyval)

#else	/* !MULTIPLICITY */

#define PL_IArgv		PL_Argv
#define PL_ICmd			PL_Cmd
#define PL_IDBcv		PL_DBcv
#define PL_IDBgv		PL_DBgv
#define PL_IDBline		PL_DBline
#define PL_IDBsignal		PL_DBsignal
#define PL_IDBsingle		PL_DBsingle
#define PL_IDBsub		PL_DBsub
#define PL_IDBtrace		PL_DBtrace
#define PL_IDir			PL_Dir
#define PL_IEnv			PL_Env
#define PL_ILIO			PL_LIO
#define PL_IMem			PL_Mem
#define PL_IProc		PL_Proc
#define PL_ISock		PL_Sock
#define PL_IStdIO		PL_StdIO
#define PL_Iamagic_generation	PL_amagic_generation
#define PL_Iampergv		PL_ampergv
#define PL_Ian			PL_an
#define PL_Iarchpat_auto	PL_archpat_auto
#define PL_Iargvgv		PL_argvgv
#define PL_Iargvoutgv		PL_argvoutgv
#define PL_Ibasetime		PL_basetime
#define PL_Ibeginav		PL_beginav
#define PL_Ibitcount		PL_bitcount
#define PL_Ibufend		PL_bufend
#define PL_Ibufptr		PL_bufptr
#define PL_Icddir		PL_cddir
#define PL_Icollation_ix	PL_collation_ix
#define PL_Icollation_name	PL_collation_name
#define PL_Icollation_standard	PL_collation_standard
#define PL_Icollxfrm_base	PL_collxfrm_base
#define PL_Icollxfrm_mult	PL_collxfrm_mult
#define PL_Icompcv		PL_compcv
#define PL_Icompiling		PL_compiling
#define PL_Icomppad		PL_comppad
#define PL_Icomppad_name	PL_comppad_name
#define PL_Icomppad_name_fill	PL_comppad_name_fill
#define PL_Icomppad_name_floor	PL_comppad_name_floor
#define PL_Icop_seqmax		PL_cop_seqmax
#define PL_Icopline		PL_copline
#define PL_Icred_mutex		PL_cred_mutex
#define PL_Icryptseen		PL_cryptseen
#define PL_Icshlen		PL_cshlen
#define PL_Icshname		PL_cshname
#define PL_Icurcopdb		PL_curcopdb
#define PL_Icurstname		PL_curstname
#define PL_Icurthr		PL_curthr
#define PL_Idbargs		PL_dbargs
#define PL_Idebdelim		PL_debdelim
#define PL_Idebname		PL_debname
#define PL_Idebstash		PL_debstash
#define PL_Idebug		PL_debug
#define PL_Idefgv		PL_defgv
#define PL_Idiehook		PL_diehook
#define PL_Idlevel		PL_dlevel
#define PL_Idlmax		PL_dlmax
#define PL_Idoextract		PL_doextract
#define PL_Idoswitches		PL_doswitches
#define PL_Idowarn		PL_dowarn
#define PL_Ie_script		PL_e_script
#define PL_Iefloatbuf		PL_efloatbuf
#define PL_Iefloatsize		PL_efloatsize
#define PL_Iegid		PL_egid
#define PL_Iendav		PL_endav
#define PL_Ienvgv		PL_envgv
#define PL_Ierrgv		PL_errgv
#define PL_Ierror_count		PL_error_count
#define PL_Ieuid		PL_euid
#define PL_Ieval_cond		PL_eval_cond
#define PL_Ieval_mutex		PL_eval_mutex
#define PL_Ieval_owner		PL_eval_owner
#define PL_Ieval_root		PL_eval_root
#define PL_Ieval_start		PL_eval_start
#define PL_Ievalseq		PL_evalseq
#define PL_Iexitlist		PL_exitlist
#define PL_Iexitlistlen		PL_exitlistlen
#define PL_Iexpect		PL_expect
#define PL_Ifdpid		PL_fdpid
#define PL_Ifilemode		PL_filemode
#define PL_Ifilter_debug	PL_filter_debug
#define PL_Iforkprocess		PL_forkprocess
#define PL_Iformfeed		PL_formfeed
#define PL_Igeneration		PL_generation
#define PL_Igensym		PL_gensym
#define PL_Igid			PL_gid
#define PL_Iglob_index		PL_glob_index
#define PL_Iglobalstash		PL_globalstash
#define PL_Ihe_root		PL_he_root
#define PL_Ihintgv		PL_hintgv
#define PL_Ihints		PL_hints
#define PL_Iin_clean_all	PL_in_clean_all
#define PL_Iin_clean_objs	PL_in_clean_objs
#define PL_Iin_my		PL_in_my
#define PL_Iin_my_stash		PL_in_my_stash
#define PL_Iincgv		PL_incgv
#define PL_Iinitav		PL_initav
#define PL_Iinplace		PL_inplace
#define PL_Ilast_lop		PL_last_lop
#define PL_Ilast_lop_op		PL_last_lop_op
#define PL_Ilast_swash_hv	PL_last_swash_hv
#define PL_Ilast_swash_key	PL_last_swash_key
#define PL_Ilast_swash_klen	PL_last_swash_klen
#define PL_Ilast_swash_slen	PL_last_swash_slen
#define PL_Ilast_swash_tmps	PL_last_swash_tmps
#define PL_Ilast_uni		PL_last_uni
#define PL_Ilastfd		PL_lastfd
#define PL_Ilastsize		PL_lastsize
#define PL_Ilastspbase		PL_lastspbase
#define PL_Ilaststatval		PL_laststatval
#define PL_Ilaststype		PL_laststype
#define PL_Ileftgv		PL_leftgv
#define PL_Ilex_brackets	PL_lex_brackets
#define PL_Ilex_brackstack	PL_lex_brackstack
#define PL_Ilex_casemods	PL_lex_casemods
#define PL_Ilex_casestack	PL_lex_casestack
#define PL_Ilex_defer		PL_lex_defer
#define PL_Ilex_dojoin		PL_lex_dojoin
#define PL_Ilex_expect		PL_lex_expect
#define PL_Ilex_fakebrack	PL_lex_fakebrack
#define PL_Ilex_formbrack	PL_lex_formbrack
#define PL_Ilex_inpat		PL_lex_inpat
#define PL_Ilex_inwhat		PL_lex_inwhat
#define PL_Ilex_op		PL_lex_op
#define PL_Ilex_repl		PL_lex_repl
#define PL_Ilex_starts		PL_lex_starts
#define PL_Ilex_state		PL_lex_state
#define PL_Ilex_stuff		PL_lex_stuff
#define PL_Ilineary		PL_lineary
#define PL_Ilinestart		PL_linestart
#define PL_Ilinestr		PL_linestr
#define PL_Ilocalpatches	PL_localpatches
#define PL_Imain_cv		PL_main_cv
#define PL_Imain_root		PL_main_root
#define PL_Imain_start		PL_main_start
#define PL_Imalloc_mutex	PL_malloc_mutex
#define PL_Imax_intro_pending	PL_max_intro_pending
#define PL_Imaxo		PL_maxo
#define PL_Imaxsysfd		PL_maxsysfd
#define PL_Imess_sv		PL_mess_sv
#define PL_Imin_intro_pending	PL_min_intro_pending
#define PL_Iminus_F		PL_minus_F
#define PL_Iminus_a		PL_minus_a
#define PL_Iminus_c		PL_minus_c
#define PL_Iminus_l		PL_minus_l
#define PL_Iminus_n		PL_minus_n
#define PL_Iminus_p		PL_minus_p
#define PL_Imodglobal		PL_modglobal
#define PL_Imulti_close		PL_multi_close
#define PL_Imulti_end		PL_multi_end
#define PL_Imulti_open		PL_multi_open
#define PL_Imulti_start		PL_multi_start
#define PL_Imultiline		PL_multiline
#define PL_Imystrk		PL_mystrk
#define PL_Inexttoke		PL_nexttoke
#define PL_Inexttype		PL_nexttype
#define PL_Inextval		PL_nextval
#define PL_Inice_chunk		PL_nice_chunk
#define PL_Inice_chunk_size	PL_nice_chunk_size
#define PL_Inomemok		PL_nomemok
#define PL_Inthreads		PL_nthreads
#define PL_Inthreads_cond	PL_nthreads_cond
#define PL_Inumeric_local	PL_numeric_local
#define PL_Inumeric_name	PL_numeric_name
#define PL_Inumeric_radix	PL_numeric_radix
#define PL_Inumeric_standard	PL_numeric_standard
#define PL_Iofmt		PL_ofmt
#define PL_Ioldbufptr		PL_oldbufptr
#define PL_Ioldlastpm		PL_oldlastpm
#define PL_Ioldname		PL_oldname
#define PL_Ioldoldbufptr	PL_oldoldbufptr
#define PL_Iop_mask		PL_op_mask
#define PL_Iop_seqmax		PL_op_seqmax
#define PL_Iorigalen		PL_origalen
#define PL_Iorigargc		PL_origargc
#define PL_Iorigargv		PL_origargv
#define PL_Iorigenviron		PL_origenviron
#define PL_Iorigfilename	PL_origfilename
#define PL_Iors			PL_ors
#define PL_Iorslen		PL_orslen
#define PL_Iosname		PL_osname
#define PL_Ipad_reset_pending	PL_pad_reset_pending
#define PL_Ipadix		PL_padix
#define PL_Ipadix_floor		PL_padix_floor
#define PL_Iparsehook		PL_parsehook
#define PL_Ipatchlevel		PL_patchlevel
#define PL_Ipending_ident	PL_pending_ident
#define PL_Iperl_destruct_level	PL_perl_destruct_level
#define PL_Iperldb		PL_perldb
#define PL_Ipidstatus		PL_pidstatus
#define PL_Ipreambleav		PL_preambleav
#define PL_Ipreambled		PL_preambled
#define PL_Ipreprocess		PL_preprocess
#define PL_Iprofiledata		PL_profiledata
#define PL_Ireplgv		PL_replgv
#define PL_Irightgv		PL_rightgv
#define PL_Irsfp		PL_rsfp
#define PL_Irsfp_filters	PL_rsfp_filters
#define PL_Irunops		PL_runops
#define PL_Isawampersand	PL_sawampersand
#define PL_Isawstudy		PL_sawstudy
#define PL_Isawvec		PL_sawvec
#define PL_Ish_path		PL_sh_path
#define PL_Isiggv		PL_siggv
#define PL_Isighandlerp		PL_sighandlerp
#define PL_Isplitstr		PL_splitstr
#define PL_Isrand_called	PL_srand_called
#define PL_Istatusvalue		PL_statusvalue
#define PL_Istatusvalue_vms	PL_statusvalue_vms
#define PL_Istdingv		PL_stdingv
#define PL_Istrchop		PL_strchop
#define PL_Istrtab		PL_strtab
#define PL_Istrtab_mutex	PL_strtab_mutex
#define PL_Isub_generation	PL_sub_generation
#define PL_Isublex_info		PL_sublex_info
#define PL_Isubline		PL_subline
#define PL_Isubname		PL_subname
#define PL_Isv_arenaroot	PL_sv_arenaroot
#define PL_Isv_count		PL_sv_count
#define PL_Isv_mutex		PL_sv_mutex
#define PL_Isv_no		PL_sv_no
#define PL_Isv_objcount		PL_sv_objcount
#define PL_Isv_root		PL_sv_root
#define PL_Isv_undef		PL_sv_undef
#define PL_Isv_yes		PL_sv_yes
#define PL_Isvref_mutex		PL_svref_mutex
#define PL_Isys_intern		PL_sys_intern
#define PL_Itainting		PL_tainting
#define PL_Ithisexpr		PL_thisexpr
#define PL_Ithr_key		PL_thr_key
#define PL_Ithreadnum		PL_threadnum
#define PL_Ithreads_mutex	PL_threads_mutex
#define PL_Ithreadsv_names	PL_threadsv_names
#define PL_Ithrsv		PL_thrsv
#define PL_Itokenbuf		PL_tokenbuf
#define PL_Iuid			PL_uid
#define PL_Iunsafe		PL_unsafe
#define PL_Iutf8_alnum		PL_utf8_alnum
#define PL_Iutf8_alnumc		PL_utf8_alnumc
#define PL_Iutf8_alpha		PL_utf8_alpha
#define PL_Iutf8_ascii		PL_utf8_ascii
#define PL_Iutf8_cntrl		PL_utf8_cntrl
#define PL_Iutf8_digit		PL_utf8_digit
#define PL_Iutf8_graph		PL_utf8_graph
#define PL_Iutf8_lower		PL_utf8_lower
#define PL_Iutf8_mark		PL_utf8_mark
#define PL_Iutf8_print		PL_utf8_print
#define PL_Iutf8_punct		PL_utf8_punct
#define PL_Iutf8_space		PL_utf8_space
#define PL_Iutf8_tolower	PL_utf8_tolower
#define PL_Iutf8_totitle	PL_utf8_totitle
#define PL_Iutf8_toupper	PL_utf8_toupper
#define PL_Iutf8_upper		PL_utf8_upper
#define PL_Iutf8_xdigit		PL_utf8_xdigit
#define PL_Iuudmap		PL_uudmap
#define PL_Iwarnhook		PL_warnhook
#define PL_Ixiv_arenaroot	PL_xiv_arenaroot
#define PL_Ixiv_root		PL_xiv_root
#define PL_Ixnv_root		PL_xnv_root
#define PL_Ixpv_root		PL_xpv_root
#define PL_Ixrv_root		PL_xrv_root
#define PL_Iyychar		PL_yychar
#define PL_Iyydebug		PL_yydebug
#define PL_Iyyerrflag		PL_yyerrflag
#define PL_Iyylval		PL_yylval
#define PL_Iyynerrs		PL_yynerrs
#define PL_Iyyval		PL_yyval

#ifndef USE_THREADS

#define PL_TSv			PL_Sv
#define PL_TXpv			PL_Xpv
#define PL_Tav_fetch_sv		PL_av_fetch_sv
#define PL_Tbodytarget		PL_bodytarget
#define PL_Tbostr		PL_bostr
#define PL_Tchopset		PL_chopset
#define PL_Tcolors		PL_colors
#define PL_Tcolorset		PL_colorset
#define PL_Tcurcop		PL_curcop
#define PL_Tcurpad		PL_curpad
#define PL_Tcurpm		PL_curpm
#define PL_Tcurstack		PL_curstack
#define PL_Tcurstackinfo	PL_curstackinfo
#define PL_Tcurstash		PL_curstash
#define PL_Tdefoutgv		PL_defoutgv
#define PL_Tdefstash		PL_defstash
#define PL_Tdelaymagic		PL_delaymagic
#define PL_Tdirty		PL_dirty
#define PL_Tdumpindent		PL_dumpindent
#define PL_Textralen		PL_extralen
#define PL_Tfirstgv		PL_firstgv
#define PL_Tformtarget		PL_formtarget
#define PL_Thv_fetch_ent_mh	PL_hv_fetch_ent_mh
#define PL_Thv_fetch_sv		PL_hv_fetch_sv
#define PL_Tin_eval		PL_in_eval
#define PL_Tlast_in_gv		PL_last_in_gv
#define PL_Tlastgotoprobe	PL_lastgotoprobe
#define PL_Tlastscream		PL_lastscream
#define PL_Tlocalizing		PL_localizing
#define PL_Tmainstack		PL_mainstack
#define PL_Tmarkstack		PL_markstack
#define PL_Tmarkstack_max	PL_markstack_max
#define PL_Tmarkstack_ptr	PL_markstack_ptr
#define PL_Tmaxscream		PL_maxscream
#define PL_Tmodcount		PL_modcount
#define PL_Tna			PL_na
#define PL_Tnrs			PL_nrs
#define PL_Tofs			PL_ofs
#define PL_Tofslen		PL_ofslen
#define PL_Top			PL_op
#define PL_Topsave		PL_opsave
#define PL_Tprotect		PL_protect
#define PL_Treg_call_cc		PL_reg_call_cc
#define PL_Treg_curpm		PL_reg_curpm
#define PL_Treg_eval_set	PL_reg_eval_set
#define PL_Treg_flags		PL_reg_flags
#define PL_Treg_ganch		PL_reg_ganch
#define PL_Treg_leftiter	PL_reg_leftiter
#define PL_Treg_magic		PL_reg_magic
#define PL_Treg_maxiter		PL_reg_maxiter
#define PL_Treg_oldcurpm	PL_reg_oldcurpm
#define PL_Treg_oldpos		PL_reg_oldpos
#define PL_Treg_oldsaved	PL_reg_oldsaved
#define PL_Treg_oldsavedlen	PL_reg_oldsavedlen
#define PL_Treg_poscache	PL_reg_poscache
#define PL_Treg_poscache_size	PL_reg_poscache_size
#define PL_Treg_re		PL_reg_re
#define PL_Treg_start_tmp	PL_reg_start_tmp
#define PL_Treg_start_tmpl	PL_reg_start_tmpl
#define PL_Treg_starttry	PL_reg_starttry
#define PL_Treg_sv		PL_reg_sv
#define PL_Treg_whilem_seen	PL_reg_whilem_seen
#define PL_Tregbol		PL_regbol
#define PL_Tregcc		PL_regcc
#define PL_Tregcode		PL_regcode
#define PL_Tregcomp_parse	PL_regcomp_parse
#define PL_Tregcomp_rx		PL_regcomp_rx
#define PL_Tregcompp		PL_regcompp
#define PL_Tregdata		PL_regdata
#define PL_Tregdummy		PL_regdummy
#define PL_Tregendp		PL_regendp
#define PL_Tregeol		PL_regeol
#define PL_Tregexecp		PL_regexecp
#define PL_Tregflags		PL_regflags
#define PL_Tregfree		PL_regfree
#define PL_Tregindent		PL_regindent
#define PL_Treginput		PL_reginput
#define PL_Tregint_start	PL_regint_start
#define PL_Tregint_string	PL_regint_string
#define PL_Treginterp_cnt	PL_reginterp_cnt
#define PL_Treglastparen	PL_reglastparen
#define PL_Tregnarrate		PL_regnarrate
#define PL_Tregnaughty		PL_regnaughty
#define PL_Tregnpar		PL_regnpar
#define PL_Tregprecomp		PL_regprecomp
#define PL_Tregprev		PL_regprev
#define PL_Tregprogram		PL_regprogram
#define PL_Tregsawback		PL_regsawback
#define PL_Tregseen		PL_regseen
#define PL_Tregsize		PL_regsize
#define PL_Tregstartp		PL_regstartp
#define PL_Tregtill		PL_regtill
#define PL_Tregxend		PL_regxend
#define PL_Trestartop		PL_restartop
#define PL_Tretstack		PL_retstack
#define PL_Tretstack_ix		PL_retstack_ix
#define PL_Tretstack_max	PL_retstack_max
#define PL_Trs			PL_rs
#define PL_Tsavestack		PL_savestack
#define PL_Tsavestack_ix	PL_savestack_ix
#define PL_Tsavestack_max	PL_savestack_max
#define PL_Tscopestack		PL_scopestack
#define PL_Tscopestack_ix	PL_scopestack_ix
#define PL_Tscopestack_max	PL_scopestack_max
#define PL_Tscreamfirst		PL_screamfirst
#define PL_Tscreamnext		PL_screamnext
#define PL_Tsecondgv		PL_secondgv
#define PL_Tseen_evals		PL_seen_evals
#define PL_Tseen_zerolen	PL_seen_zerolen
#define PL_Tsortcop		PL_sortcop
#define PL_Tsortcxix		PL_sortcxix
#define PL_Tsortstash		PL_sortstash
#define PL_Tstack_base		PL_stack_base
#define PL_Tstack_max		PL_stack_max
#define PL_Tstack_sp		PL_stack_sp
#define PL_Tstart_env		PL_start_env
#define PL_Tstatbuf		PL_statbuf
#define PL_Tstatcache		PL_statcache
#define PL_Tstatgv		PL_statgv
#define PL_Tstatname		PL_statname
#define PL_Ttainted		PL_tainted
#define PL_Ttimesbuf		PL_timesbuf
#define PL_Ttmps_floor		PL_tmps_floor
#define PL_Ttmps_ix		PL_tmps_ix
#define PL_Ttmps_max		PL_tmps_max
#define PL_Ttmps_stack		PL_tmps_stack
#define PL_Ttop_env		PL_top_env
#define PL_Ttoptarget		PL_toptarget
#define PL_Twatchaddr		PL_watchaddr
#define PL_Twatchok		PL_watchok

#endif /* USE_THREADS */

/* Hide what would have been interpreter-specific symbols? */


#ifndef USE_THREADS


#endif /* USE_THREADS */
#endif /* MULTIPLICITY */

/* Now same trickey for per-thread variables */

#ifdef USE_THREADS

#define PL_Sv			(thr->TSv)
#define PL_Xpv			(thr->TXpv)
#define PL_av_fetch_sv		(thr->Tav_fetch_sv)
#define PL_bodytarget		(thr->Tbodytarget)
#define PL_bostr		(thr->Tbostr)
#define PL_chopset		(thr->Tchopset)
#define PL_colors		(thr->Tcolors)
#define PL_colorset		(thr->Tcolorset)
#define PL_curcop		(thr->Tcurcop)
#define PL_curpad		(thr->Tcurpad)
#define PL_curpm		(thr->Tcurpm)
#define PL_curstack		(thr->Tcurstack)
#define PL_curstackinfo		(thr->Tcurstackinfo)
#define PL_curstash		(thr->Tcurstash)
#define PL_defoutgv		(thr->Tdefoutgv)
#define PL_defstash		(thr->Tdefstash)
#define PL_delaymagic		(thr->Tdelaymagic)
#define PL_dirty		(thr->Tdirty)
#define PL_dumpindent		(thr->Tdumpindent)
#define PL_extralen		(thr->Textralen)
#define PL_firstgv		(thr->Tfirstgv)
#define PL_formtarget		(thr->Tformtarget)
#define PL_hv_fetch_ent_mh	(thr->Thv_fetch_ent_mh)
#define PL_hv_fetch_sv		(thr->Thv_fetch_sv)
#define PL_in_eval		(thr->Tin_eval)
#define PL_last_in_gv		(thr->Tlast_in_gv)
#define PL_lastgotoprobe	(thr->Tlastgotoprobe)
#define PL_lastscream		(thr->Tlastscream)
#define PL_localizing		(thr->Tlocalizing)
#define PL_mainstack		(thr->Tmainstack)
#define PL_markstack		(thr->Tmarkstack)
#define PL_markstack_max	(thr->Tmarkstack_max)
#define PL_markstack_ptr	(thr->Tmarkstack_ptr)
#define PL_maxscream		(thr->Tmaxscream)
#define PL_modcount		(thr->Tmodcount)
#define PL_na			(thr->Tna)
#define PL_nrs			(thr->Tnrs)
#define PL_ofs			(thr->Tofs)
#define PL_ofslen		(thr->Tofslen)
#define PL_op			(thr->Top)
#define PL_opsave		(thr->Topsave)
#define PL_protect		(thr->Tprotect)
#define PL_reg_call_cc		(thr->Treg_call_cc)
#define PL_reg_curpm		(thr->Treg_curpm)
#define PL_reg_eval_set		(thr->Treg_eval_set)
#define PL_reg_flags		(thr->Treg_flags)
#define PL_reg_ganch		(thr->Treg_ganch)
#define PL_reg_leftiter		(thr->Treg_leftiter)
#define PL_reg_magic		(thr->Treg_magic)
#define PL_reg_maxiter		(thr->Treg_maxiter)
#define PL_reg_oldcurpm		(thr->Treg_oldcurpm)
#define PL_reg_oldpos		(thr->Treg_oldpos)
#define PL_reg_oldsaved		(thr->Treg_oldsaved)
#define PL_reg_oldsavedlen	(thr->Treg_oldsavedlen)
#define PL_reg_poscache		(thr->Treg_poscache)
#define PL_reg_poscache_size	(thr->Treg_poscache_size)
#define PL_reg_re		(thr->Treg_re)
#define PL_reg_start_tmp	(thr->Treg_start_tmp)
#define PL_reg_start_tmpl	(thr->Treg_start_tmpl)
#define PL_reg_starttry		(thr->Treg_starttry)
#define PL_reg_sv		(thr->Treg_sv)
#define PL_reg_whilem_seen	(thr->Treg_whilem_seen)
#define PL_regbol		(thr->Tregbol)
#define PL_regcc		(thr->Tregcc)
#define PL_regcode		(thr->Tregcode)
#define PL_regcomp_parse	(thr->Tregcomp_parse)
#define PL_regcomp_rx		(thr->Tregcomp_rx)
#define PL_regcompp		(thr->Tregcompp)
#define PL_regdata		(thr->Tregdata)
#define PL_regdummy		(thr->Tregdummy)
#define PL_regendp		(thr->Tregendp)
#define PL_regeol		(thr->Tregeol)
#define PL_regexecp		(thr->Tregexecp)
#define PL_regflags		(thr->Tregflags)
#define PL_regfree		(thr->Tregfree)
#define PL_regindent		(thr->Tregindent)
#define PL_reginput		(thr->Treginput)
#define PL_regint_start		(thr->Tregint_start)
#define PL_regint_string	(thr->Tregint_string)
#define PL_reginterp_cnt	(thr->Treginterp_cnt)
#define PL_reglastparen		(thr->Treglastparen)
#define PL_regnarrate		(thr->Tregnarrate)
#define PL_regnaughty		(thr->Tregnaughty)
#define PL_regnpar		(thr->Tregnpar)
#define PL_regprecomp		(thr->Tregprecomp)
#define PL_regprev		(thr->Tregprev)
#define PL_regprogram		(thr->Tregprogram)
#define PL_regsawback		(thr->Tregsawback)
#define PL_regseen		(thr->Tregseen)
#define PL_regsize		(thr->Tregsize)
#define PL_regstartp		(thr->Tregstartp)
#define PL_regtill		(thr->Tregtill)
#define PL_regxend		(thr->Tregxend)
#define PL_restartop		(thr->Trestartop)
#define PL_retstack		(thr->Tretstack)
#define PL_retstack_ix		(thr->Tretstack_ix)
#define PL_retstack_max		(thr->Tretstack_max)
#define PL_rs			(thr->Trs)
#define PL_savestack		(thr->Tsavestack)
#define PL_savestack_ix		(thr->Tsavestack_ix)
#define PL_savestack_max	(thr->Tsavestack_max)
#define PL_scopestack		(thr->Tscopestack)
#define PL_scopestack_ix	(thr->Tscopestack_ix)
#define PL_scopestack_max	(thr->Tscopestack_max)
#define PL_screamfirst		(thr->Tscreamfirst)
#define PL_screamnext		(thr->Tscreamnext)
#define PL_secondgv		(thr->Tsecondgv)
#define PL_seen_evals		(thr->Tseen_evals)
#define PL_seen_zerolen		(thr->Tseen_zerolen)
#define PL_sortcop		(thr->Tsortcop)
#define PL_sortcxix		(thr->Tsortcxix)
#define PL_sortstash		(thr->Tsortstash)
#define PL_stack_base		(thr->Tstack_base)
#define PL_stack_max		(thr->Tstack_max)
#define PL_stack_sp		(thr->Tstack_sp)
#define PL_start_env		(thr->Tstart_env)
#define PL_statbuf		(thr->Tstatbuf)
#define PL_statcache		(thr->Tstatcache)
#define PL_statgv		(thr->Tstatgv)
#define PL_statname		(thr->Tstatname)
#define PL_tainted		(thr->Ttainted)
#define PL_timesbuf		(thr->Ttimesbuf)
#define PL_tmps_floor		(thr->Ttmps_floor)
#define PL_tmps_ix		(thr->Ttmps_ix)
#define PL_tmps_max		(thr->Ttmps_max)
#define PL_tmps_stack		(thr->Ttmps_stack)
#define PL_top_env		(thr->Ttop_env)
#define PL_toptarget		(thr->Ttoptarget)
#define PL_watchaddr		(thr->Twatchaddr)
#define PL_watchok		(thr->Twatchok)

#endif /* USE_THREADS */

#ifdef PERL_GLOBAL_STRUCT

#define PL_No			(PL_Vars.GNo)
#define PL_Yes			(PL_Vars.GYes)
#define PL_curinterp		(PL_Vars.Gcurinterp)
#define PL_do_undump		(PL_Vars.Gdo_undump)
#define PL_hexdigit		(PL_Vars.Ghexdigit)
#define PL_patleave		(PL_Vars.Gpatleave)

#else /* !PERL_GLOBAL_STRUCT */

#define PL_GNo			PL_No
#define PL_GYes			PL_Yes
#define PL_Gcurinterp		PL_curinterp
#define PL_Gdo_undump		PL_do_undump
#define PL_Ghexdigit		PL_hexdigit
#define PL_Gpatleave		PL_patleave


#endif /* PERL_GLOBAL_STRUCT */


#ifdef PERL_POLLUTE		/* disabled by default in 5.006 */

#define DBsingle		PL_DBsingle
#define DBsub			PL_DBsub
#define compiling		PL_compiling
#define curcop			PL_curcop
#define curstash		PL_curstash
#define debstash		PL_debstash
#define defgv			PL_defgv
#define diehook			PL_diehook
#define dirty			PL_dirty
#define dowarn			PL_dowarn
#define errgv			PL_errgv
#define na			PL_na
#define no_modify		PL_no_modify
#define perl_destruct_level	PL_perl_destruct_level
#define perldb			PL_perldb
#define rsfp			PL_rsfp
#define rsfp_filters		PL_rsfp_filters
#define stack_base		PL_stack_base
#define stack_sp		PL_stack_sp
#define stdingv			PL_stdingv
#define sv_arenaroot		PL_sv_arenaroot
#define sv_no			PL_sv_no
#define sv_undef		PL_sv_undef
#define sv_yes			PL_sv_yes
#define tainted			PL_tainted
#define tainting		PL_tainting

#endif /* PERL_POLLUTE */
